/*
 * Copyright (c) 1998 The Santa Cruz Operation, Inc.. All Rights Reserved. 
 *                                                                         
 *        THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE               
 *                   SANTA CRUZ OPERATION INC.                             
 *                                                                         
 *   The copyright notice above does not evidence any actual or intended   
 *   publication of such source code.                                      
 */

/* $Novell-NWU: $Header: /proj6/ncps/nwu_top/nws/include/sys/nemux_app.h,v 1.2 1996/04/05 22:58:17 vtag Exp $ */
/*
 * Copyright 1991, 1992 Novell, Inc. 
 * All Rights Reserved.
 *
 * This work is subject to U.S. and International copyright laws and
 * treaties.  No part of this work may be used, practiced, performed,
 * copied, distributed, revised, modified, translated, abridged,
 * condensed, expanded, collected, compiled, linked, recast,
 * transformed or adapted without the prior written consent
 * of Novell.  Any use or exploitation of this work without
 * authorization could subject the perpetrator to criminal and
 * civil liability.
 *
 */

#ifndef _NEMUX_APP_H
#define _NEMUX_APP_H

#ident "@(#)nemux_app.h	1.2"

#ifdef _KERNEL_HEADERS
#include <proc/signal.h>
#include <util/types.h>
#include <nwu/nwnet/include/nwportable.h>
#else
#include <sys/signal.h>
#include <sys/types.h>
#include "sys/nwportable.h"
#endif
/*
**	The terms DEVICE and STREAM are used below in the following way:
**		DEVICE - Stream Head or device opened by the user process
**				 to communicate with NEMUX
**		STREAM - A lower stream connected to NEMUX via the I_LINK ioctl
**
**	Constants for NEMUX follow the following conventions:
**		PS	 - Prototol Stack definitions that apply only to lower stream
**		NMXC - Constants
**		NMXD - Device State Flags
**		NMXE - Error Action Mask Definitions
**		NMXI - Ioctl Functions
**		NMXM - Multiplexor State Flags
**		NMXP - M_PROTO Message Tokens
**		NMXS - Signals
*/

/*
**	Nmx??Control_t is the structure that accompanies messages between
**	the lower stream, NEMUX, and user processes.  Data in NmxFlags and
**	NmxStreamIdx is not used by lower streams, but only by NEMUX.
**	Lower streams modules allocate the NmxInControl_t structure on
**	messages sent upstream and prepend it to messages as type M_PROTO.
**	Data in PsInfo and PsClientAddress is not used by NEMUX.
**	User processes return the NmxOutControl_t structure with all messages
**	to NEMUX as the control part of a message (M_PROTO), thus identifying
**	the lower stream and specifying engine disposition.
**
**	NmxPointer is used by NEMUX to save the pointer to the initiating device
**	when client messages are sent from an engine down stream.  NmxPointer
**	is also used when NCP timing is initiated.  It points to the timestruc_t
**	structure which NEMUX will fill as it queues a message to an engine.
**
**	Thus there are two nearly identical structures used for Nmx??Control_t.
**	These are NmxInControl_t for messages from the lan to the process, and
**	NmxOutControl_t for messages from the process to the lan.  The two
**	packets allow an optimization that removes the necessity memory
**	allocation in the kernel for the ipx headers on output messages.
**	The two structures differ only in size.  NmxInControl_t cannot exceed
**	30 bytes (IPX_HDR_SIZE).
**
**	PsMsgLength is the actual length of the data (excluding the nemux
**	control structure) sent or received.
**
**	PsInfo has different values for different message types.  Those values
**	are defined in include files specific to the message type.
**
**	Note:  Engine code is written to assume this structure is always
**	a multiple of four bytes in length, i.e., what follows will start
**	four byte aligned.
**
**	Note:  Driver code is written to assume that the client address is
**	always the last field in the NmxInControl_t.  This allows the driver
**	to avoid copying the address, as it is already in the correct position
**	in the buffer.
*/
#define PS_ADDRESS_LENGTH		12		/* Length of max size client address */
typedef struct NmxInControl {
	uint16			NmxInfo;			/* NEMUX Information */
	uint16			NmxStreamIdx;		/* NEMUX Stream Index */
	void		   *NmxPointer;			/* Pointer to kernel data */
	void		   *NmxMPT;				/* Multi_Packet Transaction Token */
	uint16			PsMsgLength;		/* Lower protocol length of message */
	uint8			PsInfo;				/* Lower protocol stack info */
	uint8			PsMsgType;			/* Lower protocol stack msg type */
	uint8			PsClientAddress[PS_ADDRESS_LENGTH]; /* other endpoints Address */
} NmxInControl_t;

/*
**	The down stream nemux header is padded at the end to equal the size
**	defined below.  This allows streams modules below nemux build appropriate
**	headers without the need to allocate a message block.  All nemux headers
**	sent to lower streams should be this size. PsData is only used by 
**  nwetc, this is the source socket when using multiple sockets per que.
**  Its placement aligns with the source socket in the ipxHeader, no copy
**  necessary by nwetc, PsData must be in net order.
*/
typedef struct NmxOutControl {
	uint16			NmxInfo;			/* NEMUX Information */
	uint16			NmxStreamIdx;		/* NEMUX Stream Index */
	void		   *NmxPointer;			/* Pointer to kernel data */
	void		   *NmxMPT;				/* Multi_Packet Transaction Token */
	uint16			PsMsgLength;		/* Lower protocol length of message */
	uint8			PsInfo;				/* Lower protocol stack info */
	uint8			PsMsgType;			/* Lower protocol stack msg type */
	uint8			PsClientAddress[PS_ADDRESS_LENGTH]; /* other endpoints Address */
	uint8			pad[2];				/* Pad to 32 bytes */
	uint16			PsData;				/* Source Socket Address, NWetc only */
} NmxOutControl_t;

/*
**	Definitions for NmxInfo
*/
enum {
	/*
	**	
	**	The NMXOutControl_t structure is sent by the process as M_PROTO.
	**	The NMXInControl_t structure is received by the process as M_DATA.
	**	Send data means send M_PROTO mblk_t to the stream below NEMUX
	**		including any M_DATA 
	**	No data means the message is discarded by NEMUX, nothing sent to stream
	**		below.
	*/
	NMXP_DATA = 1,		/* data - Nemux to engine */
						/* Engine to Nemux - data, set engine idle */
						/* NWU process to nemux - send data */
	NMXP_DATA_TIME,		/* NEMUX - Data to Engine, record time, chg to DATA */
	NMXP_EXPEDITE, 		/* lower stream to NEMUX, put msg on front of queue */
						/*  change to NMXP_DATA */
	NMXP_BUSY,			/* Engine to NEMUX, send data, don't set engine idle */
	NMXP_DONE,			/* Engine to NEMUX, no data, set engine idle */
	NMXP_NOBLOCK,	 	/* Process to NEMUX, don't block ioctl */
	NMXP_CLIENT,		/* Engine to NEMUX, client message, block until reply */
	NMXP_START_ENGINE,	/* NEMUX to NWU Daemon, start engine */
	NMXP_START_LOCKD,	/* NEMUX to NWU Daemon, start lock daemon */
	NMXP_IS_ENGINE,		/* NWU process to NEMUX, I am an engine */
	NMXP_IS_ANCILLARY,	/* NWU process to NEMUX, I'm a nwu ancillary process */
	NMXP_SHUTDOWN,		/* NEMUX to Engine, do shutdown */
						/* Lock Daemon to NEMUX, shutting down process */
	NMXP_EXIT,			/* NEMUX to Engine, exit no cleanup */
	NMXP_DOING_ERROR, 	/* NEMUX to NWU Daemon, doing err action */
	NMXP_HANGUP,		/* NEMUX to NWU Daemon, hangup received */
						/* Multi_Packet Transaction values (MPT) */
	NMXP_REQUEST_MPT,	/* Driver to NEMUX, request a MPT token */	
	NMXP_MPT_TIME,		/* Driver to Engine, record time, chg to REQUEST_MPT */
	NMXP_USE_MPT,		/* Driver to NEMUX, used engine indicated by MPT */
	NMXP_USE_EXPEDITED_MPT, /* Driver to NEMUX, same as USE_MPT, use band 1 */
	NMXP_END_MPT,		/* Engine to NEMUX, end of MPT */
						/* Driver to NEMUX, client killed, end of MPT */
	NMXP_PACKETS_QUEUED,	/* NEMUX to Engine, USE_EXPEDITED_MPT received */
						/* Set when packets queue count is non-zero */
	NMXP_NON_NWU		/* Set internally by NEMUX when a NMXP_DATA comes */
						/* downstream from a non-NWU process.  Nemux changes */
						/* the NmxInfo to NMXP_NON_NWU as an indication to */
						/* downstream modules that they should reject the */
						/* packet UNLESS it's one of the types allowed to */
						/* come from non-NWU processes.  (This is protocol- */
						/* specific: in the case of NCP, NCPX handlers are */
						/* non-NWU processes which are allowed to send */
						/* packets down in response to specific NCP requests */
						/* (NCP 37).) */
};

/*
**	Definition of message type.
**	Message type is set by protocol stack or NWU process.
**	This value determines the type of the message. 
**	PsMsgType is not used by NEMUX.
*/
#define PS_TYPE_NCP			0			/* packet is NCP */
#define PS_TYPE_AFP			1			/* packet is ASP */
#define PS_TYPE_CLIENT		2			/* packet is Client Response */
#define PS_TYPE_WATCHDOG	3			/* packet is  WatchDog from Client */
#define PS_TYPE_BROADCAST_INFORM 4		/* "message waiting" inform from a remote server. */

/*
**	The following are signals used to control the termination of NWU processes
**	Note: engines receive a shutdown request via an NmxInControl_t structure
**	received as M_DATA input, when shutdown is initiated.
*/
#define NMXS_SIG_SHUT		SIGTERM		/* Signal to shutdown non-engines */
#define NMXS_SIG_ERROR		SIGQUIT		/* Signal to kill engines */
#define NMXS_SIG_HANGUP		SIGHUP		/* Signal to indicate hangup */

#ifdef NIOC
#undef NIOC
#endif
#define NIOC(t) (('N' << 8) | t)

/*
**	The NMXI_PSI_INFO ioctl request is sent to a stream requesting module
**	statistics.  This is the only pass-through IOCTL defined by NEMUX.  Other
**	pass-through ioctl functions can be defined by the stream module.  This
**	and other pass-through IOCTL functions are used by doing an IOCTL using
**	the NMXI_IOCTL function.
**
**	To use this function, make an I_STR ioctl call, put NMXI_IOCTL in ic_cmd
**	of the strioctl structure.  A NmxiIoctl_t structure is used to with this
**	ioctl function.  The actual ioctl command to be interpreted by the stream
**	module, e.g. NMXI_PSI_INFO, is placed in PsIoctlCmd by the user.
**	The PsClientAddress is set to the client (if any) to which the ioctl
**	is applied.  NmxStreamIdx is set to identify the appropriate lower stream.
**	The data structure specific to the ioctl sent to the stream begins at
**	PsIoctlData. A data structure containing statistics specific to 
**	the stream is returned to the requester if a NMXI_PSI_INFO
**	ioctl was requested.
*/
#define	NMXI_PSI_INFO		NIOC(20)	/* Pass IOCTL to stream, get info */
#define	NMXI_INITIALIZE		NIOC(21)	/* Daemon is ready to start engines */

/*
**	Other I_STR IOCTL functions recognized by the Engine Multiplexer
**	NMXI_ABORT is also recognized by NEMUX comming upstream and causes
**	immediate abort (INTERNAL_ERROR) action to happen.  This allows down 
**	stream modulesto initiate error conditions.  It is sent up as an M_CTL 
**	with anormal ioctl packet.  Upstream M_CTL messages are not ack'd.
*/
#define	NMXI_SET_ENGINE_CNT		NIOC(2)	/* Set number of engines */
#define	NMXI_PROCESS_IDENTITY	NIOC(3)	/* Indicate new process identity */
#define	NMXI_SHUTDOWN			NIOC(4)	/* Shut down the NWU */
#define	NMXI_SET_ERROR_DFLT		NIOC(5)	/* Set default action for errors */
#define	NMXI_ABORT				NIOC(6)	/* Perform default error action now */
#define	NMXI_DO_ERROR			NIOC(7)	/* Perform a specified error action */
#define	NMXI_IOCTL				NIOC(8)	/* IOCTL request to specified stream */
#define	NMXI_NAME				NIOC(9)	/* Set name of lower stream */
#define	NMXI_INFO				NIOC(10)/* Request NEMUX stat information */
#define	NMXI_DUMP				NIOC(11)/* Request NEMUX device & stream stat */
#define	NMXI_START_LOCK_DAEMON	NIOC(12)/* Start lock daemon */
#define	NMXI_DATA				NIOC(13)/* Used by ioctls that need to send */
										/*   M_DATA blocks to requester */

/*
**	IOCTL data structure for NMXI_SET_ENGINE_CNT
*/
typedef struct {
	int16			NumEngines;			/* Max Number of Engines */
	int16			Mask;				/* Error Action Mask */
} NmxiInitialize_t;

/*
**	Defintions for Error Actions set in Mask
**
**	IOCTL data structure for NMXI_IOCTL
*/
typedef struct {
	uint16			 Reserved;
	uint16			 NmxStreamIdx;	/* NEMUX Stream Index */
	int				 PsIoctlCmd;	/* Ioctl command for stream */
	uint8			 PsClientAddress[PS_ADDRESS_LENGTH]; /* Dest Net Addr */
									/* Beginning of data for ioctl command */
	uint8			 pad[8];
} NmxiIoctl_t;

/*
**	IOCTL data structure for NMXI_NAME
*/
#define	NMXC_NAME_LENGTH	12
typedef struct {

	int				StreamId;				/* Stream Identifier from I_LINK */
	char			StreamName[NMXC_NAME_LENGTH]; /* Name of lower stream */
} NmxiName_t;

/*
**	IOCTL data structure for NMXI_SET_ENGINE_CNT
*/
typedef struct {
	uint16			NmxInfo;			/* Flag - only NMXP_NOBLOCK is legal */
	int16			NumEngines;			/* Number of active engines */
} NmxiEngineCount_t;


/*
**	IOCTL data structure for NMXI_START_LOCK_DAEMON
*/
typedef struct {
	uint32			NmxProcessType;
	uint32			NmxProcessArgument1;
	uint32			NmxProcessArgument2;
} NmxiStartLockDaemon_t;

/*
**	Values for the ProcessType field
*/
enum {
	NMXC_LOGICAL_LOCK_DAEMON = 1,
	NMXC_FILE_LOCK_DAEMON,
	NMXC_BYTE_RANGE_LOCK_DAEMON,
	NMXC_SEMAPHORE_LOCK_DAEMON
};

/*
**	IOCTL data structure to set process identity and
**	sets the new pid for NWU daemon.
**		NMXP_IS_ENGINE is identifies an Engine
**		NMXP_IS_ANCILLARY is identifies non Engine NWU Processes
**	Pid identifies new NWU daemon process PID.  Set to zero if fork of
**	new process did not succeed.
*/
typedef struct {
	uint16			NmxInfo;			/* Flag */
	pid_t			NWUDaemonPid;		/* New NWU Daemon pid */
} NmxiProcessIdentity_t;

/*
**	IOCTL data structure for NMXI_SET_ERROR_DFLT & NMXI_DO_ERROR
*/
typedef struct {
	uint32			Mask;				/* Error Action Mask */
} NmxiErrorMask_t;

/*
**	Defintions for Error Actions set in Mask
**	Used with for the ioctl calls NMXI_SET_ERROR_DFLT & NMXI_DO_ERROR
**	DO_ERROR actions force the action.
**	If one of the TERM_ACTIONS are selected, the action is performed
**	but the error mask is left unchanged.  Some of the ACTIONS,
**	do not set any error conditions, but allow additional control
**	for debugging.
**	SET_ERROR_DFLT sets the action to be taken if an error occurs.
**	(marked with -)
**
**	Note:  For Error situations, either ABORT_ALL must be set
**		or ABORT_BUSY and STOP_IDLE must be set.   The default
**		error mask must a valid combination of the above set.
**
**	When NMXE_ABORT_ALL is set, all NWU processes are sent the NMXS_SIG_ERROR
**		signal when the error is detected.
**		Non NWU processes are sent NMXS_SIG_HANGUP
**
**	When NMXE_STOP_IDLE is set, all idle engines are sent a NMXP_EXIT message.
**	When NMXE_ABORT_BUSY is set, all busy engines are sent a NMXP_SIG_ERROR
**		signal.  All non engine NWU processes are sent a NMXP_SIG_ERROR signal.
**		Non NWU processes are sent a NMXP_SIG_HANUP signal.
*/
#define NMXE_STOP_NTR		0x001	/* Stop Novell NTR Trace */
#define NMXE_STOP_DATA		0x004	/* Stop all incoming messages */
#define NMXE_STOP_IDLE		0x010	/* Stop idle engines, exit no cleanup */
#define NMXE_ABORT_BUSY		0x020	/* Kill busy engines with SIG_ERROR */
#define NMXE_ABORT_ALL		0x040	/* Terminate all engines with signal */
									/* Terminate other procs with hangup */
#define NMXE_ALL_ACTIONS	0x075	/* All error actions */
									/* All termination actions */
#define NMXE_TERM_ACTIONS   (NMXE_ABORT_ALL	\
						|	NMXE_ABORT_BUSY \
						|	NMXE_STOP_IDLE) 

/*
**	Macros to check if an Error Action in mask is set
**	Value is true if set
*/
#define	ISE_STOP_DATA( Mask)		( Mask & NMXE_STOP_DATA)
#define	ISE_STOP_NTR( Mask)			( Mask & NMXE_STOP_NTR)
#define	ISE_STOP_IDLE( Mask)		( Mask & NMXE_STOP_IDLE)
#define	ISE_ABORT_BUSY( Mask)		( Mask & NMXE_ABORT_BUSY)
#define	ISE_ABORT_ALL( Mask)		( Mask & NMXE_ABORT_ALL)
#define	ISE_ALL_ACTIONS( Mask)		( Mask & NMXE_ALL_ACTIONS)
#define	ISE_TERM_ACTIONS( Mask)		( Mask & NMXE_TERM_ACTIONS)

/*
**	Macros to set an Error Action in mask
*/
#define	SETE_STOP_DATA( Mask)		(Mask |= NMXE_STOP_DATA)
#define	SETE_STOP_NTR( Mask)		(Mask |= NMXE_STOP_NTR)
#define	SETE_STOP_IDLE( Mask)		(Mask |= NMXE_STOP_IDLE)
#define	SETE_ABORT_BUSY( Mask)		(Mask |= NMXE_ABORT_BUSY)
#define	SETE_ABORT_ALL( Mask)		(Mask |= NMXE_ABORT_ALL)

/*
**	Macros to clear a Error Action in mask
*/
#define	UNSETE_STOP_DATA( Mask)		(Mask &= ~NMXE_STOP_DATA)
#define	UNSETE_STOP_NTR( Mask)		(Mask &= ~NMXE_STOP_NTR)
#define	UNSETE_STOP_IDLE( Mask)		(Mask &= ~NMXE_STOP_IDLE)
#define	UNSETE_ABORT_BUSY( Mask)	(Mask &= ~NMXE_ABORT_BUSY)
#define	UNSETE_ABORT_ALL( Mask)		(Mask &= ~NMXE_ABORT_ALL)
#define	UNSETE_ALL_ACTIONS( Mask)	(Mask &= ~NMXE_ALL_ACTIONS)

#define NMXC_INFO_BUF_SZ	1024	/* Size of data buf returnd on DUMP ioctl */
/*
**	Structure defining the data returned on NMXI_INFO or NMXI_DUMP
**	ioctl functions.  This structure is returned as data with the ioctl ACK.
*/
typedef struct {
	time_t		IStartTime;			/* Seconds since epoch */
	int16		IStreamLink;		/* Number of streams linked */
	int16		IStreamCount;		/* Number of streams strucs retnd on dump */
	dev_t		IDeviceUsed;		/* Highest device number used */
	int16		IDeviceCount;		/* Number device strucs returned on dump */
	int16		IDeviceOpen;		/* Number of devices open */
	int16		IDeviceClosed;		/* Number of devices closed */
	int16		IDeviceEngine;		/* Number of NWU engines */
	int16		IDaemon;			/* Number of NWU Daemons */
	int16		IMaxEngines;		/* Number of max NWU Engines */
	int16		IDeviceNWUProcess;	/* Number of NWU processes inclds Engines */
	int16		IEnginesRequested;	/* Number of active engines requested */
	int16		IEnginesIdle;		/* Number of idle Engines */
	int16		IEnginesDied;		/* Number of unexpected engine deaths */
	int16		INonEnginesDied;	/* Number of unexpected non-engine deaths */
	int16		INWUDaemonDied;		/* Number of unexpected NWU daemon deaths */
	int16		IEngineStartFail;	/* Number of times an engine start failed */
	int16		IUpMsgsOnQueue;		/* Number of upstream msgs on Queue */
	int16		IDownMsgsOnQueue;	/* Number of dnstream msgs on Queue */
	uint32		IUpCalls;			/* Number of up stream calls */
	uint32		IDownCalls;			/* Number of dn stream calls */
	uint32		IUpMsgsSent;		/* Number of up stream msgs sent */
	uint32		IDownMsgsSent;		/* Number of dn stream msgs sent */
	uint32		IUpIoctlResponses;	/* Number of ioctl responses from stream */
	uint32		IUpMsgsQueued;		/* Number of up stream msgs queued */
	uint32		IDownMsgsQueued;	/* Number of dn stream msgs queued */
	uint32		IUpMsgsDropped;		/* Number of up stream dropped msgs */
	uint32		IDownMsgsDropped;	/* Number of dn stream dropped */
	uint32		IUpMPTMsgs;			/* Number of up stream msgs for MPT */
	uint32		IUpMPTPackets;		/* Number of up stream packets for MPT */
	uint32		IUpClientMsgs;		/* Number of up stream msgs as a client */
	uint32		IDownClientMsgs;	/* Number of dn stream msgs as a client */
	uint32		IIoctl;				/* Number of ioctl requests */
	uint32		IState;				/* NEMUX Multiplexor State Flags */
	uint32		IErrorMask;			/* NEMUX Error Action Mask */
} NmxInfo_t;

/*
**  Definitions of NEMUX Conditions or States
*/
#define NMXM_INPUT_BLOCKED	0x00000001	/* Input is blocked */
#define NMXM_TRACE_ON		0x00000002	/* Trace is enabled */
#define NMXM_ADM_DO_SHUT	0x00000008	/* Shut down requested by admin */
#define NMXM_DMN_DO_SHUT	0x00000010	/* Shut down requested by NWU daemon */
#define NMXM_SHUT_DAEMON	0x00000020	/* Engines shutdown, shutdown daemons */
#define NMXM_STOP_IDLE_ENG	0x00000040	/* Stop idle engines requested */
#define NMXM_KILL_BUSY		0x00000080	/* Kill busy engines & NWU procsses */
#define NMXM_KILL_ALL		0x00000100	/* Kill all NWU processes */
#define NMXM_ENGINE_DIED	0x00000200	/* An engine died unexpectedly */
#define NMXM_NWU_DIED		0x00000400	/* A NWProcess died unexpectedly */
#define NMXM_CONTROL_DIED	0x00000800	/* The NWDaemon died unexpectedly */
#define NMXM_ABORT_REQUESTED 0x0001000	/* Admin Process Requested Error Abort*/
#define NMXM_INTERNAL_ERROR	0x00002000	/* NEMUX is messed up, internal error */
#define NMXM_HANGUP_RECVD	0x00004000	/* NEMUX received upstream hangup req */
#define NMXM_UNLINK			0x00008000	/* NEMUX received unlink request */
										/* When to send process a hangup */
#define NMXM_HANGUP		   (NMXM_HANGUP_RECVD		| \
							NMXM_UNLINK	 			| \
							NMXM_ABORT_REQUESTED	| \
							NMXM_INTERNAL_ERROR		| \
							NMXM_CONTROL_DIED)
										/* We are in shutdown */
#define NMXM_IN_SHUTDOWN   (NMXM_ADM_DO_SHUT	| \
							NMXM_DMN_DO_SHUT)

/*
**	Macros to test if an NEMUX state is set
**	State is IState in the NmxInfo_t structure
*/
#define	ISM_INPUT_BLOCKED( State)		( State & NMXM_INPUT_BLOCKED)
#define	ISM_TRACE_ON( State)			( State & NMXM_TRACE_ON)
#define	ISM_ADM_DO_SHUT( State)			( State & NMXM_ADM_DO_SHUT)
#define	ISM_DMN_DO_SHUT( State)			( State & NMXM_DMN_DO_SHUT)
#define	ISM_SHUT_DAEMON( State)			( State & NMXM_SHUT_DAEMON)
#define	ISM_STOP_IDLE_ENG( State)		( State & NMXM_STOP_IDLE_ENG)
#define	ISM_KILL_BUSY( State)			( State & NMXM_KILL_BUSY)
#define	ISM_KILL_ALL( State)			( State & NMXM_KILL_ALL)
#define	ISM_ENGINE_DIED( State)			( State & NMXM_ENGINE_DIED)
#define	ISM_NWU_DIED( State)			( State & NMXM_NWU_DIED)
#define	ISM_CONTROL_DIED( State)		( State & NMXM_CONTROL_DIED)
#define	ISM_ABORT_REQUESTED( State)		( State & NMXM_ABORT_REQUESTED)
#define	ISM_INTERNAL_ERROR( State)		( State & NMXM_INTERNAL_ERROR)
#define	ISM_HANGUP_RECVD( State)		( State & NMXM_HANGUP_RECVD)
#define	ISM_UNLINK( State)				( State & NMXM_UNLINK)
#define	ISM_HANGUP( State)				( State & NMXM_HANGUP)
#define	ISM_IN_SHUTDOWN( State)			( State & NMXM_IN_SHUTDOWN)

/*
**	Structure defining the stream data returned on a NMXI_DUMP ioctl.
**	This data is obtained by doing a read following the ioctl ACK.
**	This represents the data for a single lower multiplexor instance
*/
typedef struct {
	int16		 SStreamIndex;		/* Stream Index */
	int16		 SUpMsgsOnQueue;	/* # upstream msgs currently on queue */
	int16		 SDownMsgsOnQueue;	/* # dnstream msgs currently on queue */
	uint32		 SUpMsgsSent;		/* # msgs passed directly upstream */
	uint32		 SDownMsgsSent;		/* # msgs sent directly down stream */
	uint32		 SUpMsgsQueued;		/* # upstream messages queued */
	uint32		 SDownMsgsQueued;	/* # dnstream msgs queued */
	uint32		 SUpMsgsDropped;	/* # unknown or bad upStream msgs dropped */
	uint32		 SIoctlRequest;		/* # ioctl requests sent down stream */
	uint32		 SIoctlResponse;	/* # ioctl responses received up stream */
	int			 SIoctlCommand;		/* Last ioctl command sent down stream */
	dev_t		 SLastDevice;		/* Device last msg sent to */
	char		 SName[NMXC_NAME_LENGTH]; /* Name of this stream */
} NmxStreamInfo_t;

/*
**	Structure defining the data returned on a NMXI_DUMP ioctl.
**	This data is obtained by doing a read following the ioctl ACK.
**	This represents the data for a single minor device instance
*/
typedef struct {
	dev_t		 DMinorDevice;		/* Minor device number */
	pid_t		 DProcessPid;		/* PID of process */
	uint16		 DProcessState;		/* Flags showing state of device */
	int16		 DDnStreamIdx;		/* Stream Index where last msg sent */
	int16		 DUpStreamIdx;		/* Stream Index last msg received from */
	uint16		 DDnNmxInfo;		/* NmxInfo of last message sent */
	uint8		 DDnPsInfo;			/* PsInfo of last message sent */
	uint8		 DDnPsType;			/* PsMsgType of last message sent */
	uint16		 DUpNmxInfo;		/* NmxInfo for lest message received */
	uint8		 DUpPsInfo;			/* PsInfo of last message sent */
	uint8		 DUpPsType;			/* PsMsgType of last message sent */
	uint32		 DDownCalls;		/* # calls from process */
	uint32		 DUpMsgsDequeued;	/* # msgs dequeued and passed to process */
	uint32		 DUpMsgsSent;		/* # msgs sent to process w/o queueing */
	uint32		 DDownMsgsSent;		/* # msgs passed down stream */
	uint32		 DDownMsgsDropped;	/* # msgs down stream dropped */
	uint32		 DMPTInProgress;	/* # MPT requests in progress */
	uint32		 DPacketsQueued;	/* # Number of packets currently on queue */
	uint32		 DIoctlRequest;		/* # ioctl requests sent */
	uint32		 DIoctlResponse;	/* # ioctl responses recvd  */
	int			 DIoctlCmd;			/* last ioctl command sent down stream */
}  NmxDeviceInfo_t;

/*
**	Flags for Device State
*/
#define	NMXD_OPEN		0x00000001	/* Device is open */
#define	NMXD_CTL		0x00000002	/* Device is NWU Daemon, control device */
#define	NMXD_PROCESS	0x00000004	/* Device is a NWU process */
#define	NMXD_ENGINE		0x00000008	/* Device is a NWU engine */
#define	NMXD_BUSY		0x00000010	/* Engine is busy */
#define NMXD_IMUNE		0x00000020	/* Process is imune from error action */
#define NMXD_SICK		0x00000040	/* NWU process sending wierd data */
#define	NMXD_SHUT		0x00000080	/* Shutdown request sent to process */
#define	NMXD_EXIT		0x00000100	/* Exit request to engine, no cleanup */
#define	NMXD_SIG		0x00000200	/* SIG_ERROR sent to process */
#define NMXD_HUP		0x00000400	/* Hangup sent to process */
#define NMXD_DIED		0x00000800	/* Process terminated abnormally */
									/* Process terminate reqst sent or imune */
#define NMXD_TERM		(NMXD_IMUNE	\
					|	 NMXD_DIED	\
					|	 NMXD_HUP	\
					|	 NMXD_EXIT	\
					|	 NMXD_SIG	\
					|	 NMXD_SHUT)

/*
**	Macros to check device state
**	DInfo is a pointer to the NmxDeviceInfo_t structure
*/
#define	ISD_OPEN( DInfo)	( DInfo->DProcessState & NMXD_OPEN)
#define ISD_CTL( DInfo)		( DInfo->DProcessState & NMXD_CTL)
#define ISD_PROCESS( DInfo)	( DInfo->DProcessState & NMXD_PROCESS)
#define ISD_ENGINE( DInfo)	( DInfo->DProcessState & NMXD_ENGINE)
#define ISD_BUSY( DInfo)	( DInfo->DProcessState & NMXD_BUSY)
#define ISD_IMUNE( DInfo)	( DInfo->DProcessState & NMXD_IMUNE)
#define ISD_SICK( DInfo)	( DInfo->DProcessState & NMXD_SICK)
#define ISD_SHUT( DInfo)	( DInfo->DProcessState & NMXD_SHUT)
#define ISD_SIG( DInfo)		( DInfo->DProcessState & NMXD_SIG)
#define ISD_EXIT( DInfo)	( DInfo->DProcessState & NMXD_EXIT)
#define ISD_HUP( DInfo)		( DInfo->DProcessState & NMXD_HUP)
#define ISD_DIED( DInfo)	( DInfo->DProcessState & NMXD_DIED)
#define ISD_TERM( DInfo)	( DInfo->DProcessState & NMXD_TERM)

#endif	/*	_ NEMUX_APP_H */
